Imports System.Xml
Public Class LaborTextBuilder
    Private m_Gruppen As New List(Of LaborTextGruppe)

    Private m_InfarktGruppe As LaborTextGruppe
    Private m_LeberwertGruppe As LaborTextGruppe

    Private m_GerinnungsGruppe As LaborTextGruppe
    Private m_EntzndungsparameterGruppe As LaborTextGruppe

    Private m_Alat As LaborWert
    Private m_Asat As LaborWert
    Private m_Fibrinogen As LaborWert

    Private m_ActiveAlatGruppe As LaborTextGruppe
    Private m_ActiveFibrinogenGruppe As LaborTextGruppe

    Public Event Text_Changed()
    Private m_Text As String = ""
    ReadOnly Property Text() As String
        Get
            Return m_Text
        End Get
    End Property
    Public Sub CreateText()
        Dim TmpText As String = ""

        Dim PathologischTexte As New List(Of String)
        Dim NormalTexte As New List(Of String)
        For Each Grp As LaborTextGruppe In m_Gruppen
            NormalTexte.AddRange(Grp.NormalTexte.ToArray)
            PathologischTexte.AddRange(Grp.PathologischTexte.ToArray)
        Next

        Dim PathologischText As String = JoinToText(PathologischTexte)
        Dim NormalText As String = JoinToText(NormalTexte)

        Dim Result As String = ""
        If PathologischTexte.Count > 0 Then
            Result = "Pathologisch waren " & PathologischText & ". "
        End If
        If NormalTexte.Count > 0 Then
            Result &= NormalText.Substring(0, 1).ToUpper & NormalText.Substring(1) & " waren normal."
        End If

        TmpText = Result

        If m_Text <> TmpText Then
            m_Text = TmpText
            RaiseEvent Text_Changed()
        End If
    End Sub
    Private Function JoinToText(ByVal Source As List(Of String)) As String
        If Source.Count > 1 Then
            Dim TmpListe As New List(Of String)
            TmpListe.AddRange(Source.ToArray)
            Dim LastText As String = TmpListe(TmpListe.Count - 1)
            TmpListe.RemoveAt(TmpListe.Count - 1)
            Dim TmpText As String = Join(TmpListe.ToArray, ", ")
            TmpText &= " und " & LastText
            Return TmpText
        ElseIf Source.Count > 0 Then
            'Return Join(Source.ToArray, " und ")
            Return Source(0)
        Else
            Return ""
        End If
    End Function

    Public Event Changed()
    Public Sub RaiseChanged()
        RaiseEvent Changed()
    End Sub

    ReadOnly Property Gruppen() As List(Of LaborTextGruppe)
        Get
            Return m_Gruppen
        End Get
    End Property


    Private Sub CreateGroups()
        m_EntzndungsparameterGruppe = New EntzndungsParameterGruppe(Me)
        m_Gruppen.Add(m_EntzndungsparameterGruppe)
        m_Gruppen.Add(New BlutbildGruppe(Me))
        m_Gruppen.Add(New DifferentialblutbildGruppe(Me))
        m_Gruppen.Add(New ElektrolytGruppe(Me))

        m_LeberwertGruppe = New LeberwertGruppe(Me)
        m_Gruppen.Add(m_LeberwertGruppe)
        m_Gruppen.Add(New PankreasGruppe(Me))

        m_InfarktGruppe = New InfarktGruppe(Me)
        m_Gruppen.Add(m_InfarktGruppe)
        m_Gruppen.Add(New HormonGruppe(Me))
        m_Gruppen.Add(New UrinStatusGruppe(Me))

        m_GerinnungsGruppe = New GerinnungsGruppe(Me)
        m_Gruppen.Add(m_GerinnungsGruppe)
    End Sub

    Public Sub New()
        CreateGroups()
        InitMovableGroups()
    End Sub
    Public Sub New(ByVal Nd As XmlNode)
        CreateGroups()
        Dim tmpGruppen As New List(Of LaborTextGruppe)
        tmpGruppen.AddRange(m_Gruppen.ToArray)
        m_Gruppen.Clear()

        Dim Groups As New Dictionary(Of String, LaborTextGruppe)
        For Each Grp As LaborTextGruppe In tmpGruppen
            Groups.Add(Grp.Title, Grp)
        Next


        For Each xNd As XmlNode In Nd.ChildNodes
            If xNd.Name = "Werte" Then
                For Each yNd As XmlNode In xNd.ChildNodes
                    Dim Grp As LaborTextGruppe = Groups(yNd.Name)
                    Grp.SetValues(yNd)
                    m_Gruppen.Add(Grp)
                Next
                Exit For
            End If
        Next


        For Each Grp As LaborTextGruppe In tmpGruppen
            If Not m_Gruppen.Contains(Grp) Then
                m_Gruppen.Add(Grp)
            End If
        Next

        InitMovableGroups()

        CreateText()
    End Sub

    Private Sub InitMovableGroups()


        If m_Gruppen.IndexOf(m_InfarktGruppe) < m_Gruppen.IndexOf(m_LeberwertGruppe) Then
            For I As Integer = m_LeberwertGruppe.Werte.Count - 1 To 0 Step -1
                Dim Wt As LaborWert = m_LeberwertGruppe.Werte(I)
                If Wt.Kuerzel = "ALAT" OrElse Wt.Kuerzel = "ASAT" Then
                    m_LeberwertGruppe.Werte.RemoveAt(I)
                End If
            Next
            m_ActiveAlatGruppe = m_InfarktGruppe
        Else
            For I As Integer = m_InfarktGruppe.Werte.Count - 1 To 0 Step -1
                Dim Wt As LaborWert = m_InfarktGruppe.Werte(I)
                If Wt.Kuerzel = "ALAT" OrElse Wt.Kuerzel = "ASAT" Then
                    m_InfarktGruppe.Werte.RemoveAt(I)
                End If
            Next
            m_ActiveAlatGruppe = m_LeberwertGruppe
        End If

        For Each XWt As LaborWert In m_ActiveAlatGruppe.Werte
            If XWt.Kuerzel = "ALAT" Then
                m_Alat = XWt
            ElseIf XWt.Kuerzel = "ASAT" Then
                m_Asat = XWt
            End If
        Next


        If m_Gruppen.IndexOf(m_EntzndungsparameterGruppe) < m_Gruppen.IndexOf(m_GerinnungsGruppe) Then
            For I As Integer = m_GerinnungsGruppe.Werte.Count - 1 To 0 Step -1
                Dim Wt As LaborWert = m_GerinnungsGruppe.Werte(I)
                If Wt.Kuerzel = "Fib" Then
                    m_GerinnungsGruppe.Werte.RemoveAt(I)
                End If
            Next
            m_ActiveFibrinogenGruppe = m_EntzndungsparameterGruppe
        Else
            For I As Integer = m_EntzndungsparameterGruppe.Werte.Count - 1 To 0 Step -1
                Dim Wt As LaborWert = m_EntzndungsparameterGruppe.Werte(I)
                If Wt.Kuerzel = "Fib" Then
                    m_EntzndungsparameterGruppe.Werte.RemoveAt(I)
                End If
            Next
            m_ActiveFibrinogenGruppe = m_GerinnungsGruppe
        End If

        For Each XWt As LaborWert In m_ActiveFibrinogenGruppe.Werte
            If XWt.Kuerzel = "Fib" Then
                m_Fibrinogen = XWt
            End If
        Next

    End Sub


    Public Sub Write_XML(ByVal Writer As XmlTextWriter)
        Writer.WriteStartElement("Werte")
        For Each Grp As LaborTextGruppe In m_Gruppen
            Grp.Write_XML(Writer)
        Next
        Writer.WriteEndElement()
    End Sub

    Public Sub RelocateDependendLaborWerte()

        Dim NewALATGruppe As LaborTextGruppe = Nothing

        If m_Gruppen.IndexOf(m_InfarktGruppe) < m_Gruppen.IndexOf(m_LeberwertGruppe) Then
            If m_ActiveAlatGruppe IsNot m_InfarktGruppe Then
                NewALATGruppe = m_InfarktGruppe
            End If
        Else
            If m_ActiveAlatGruppe IsNot m_LeberwertGruppe Then
                NewALATGruppe = m_LeberwertGruppe
            End If
        End If

        If NewALATGruppe IsNot Nothing Then
            m_ActiveAlatGruppe.Werte.Remove(m_Alat)
            m_ActiveAlatGruppe.Werte.Remove(m_Asat)
            m_ActiveAlatGruppe.WasChanged = True
            m_ActiveAlatGruppe = NewALATGruppe

            If m_ActiveAlatGruppe Is m_LeberwertGruppe Then
                m_ActiveAlatGruppe.Werte.Insert(0, m_Asat)
                m_ActiveAlatGruppe.Werte.Insert(0, m_Alat)
            Else
                m_ActiveAlatGruppe.Werte.Add(m_Alat)
                m_ActiveAlatGruppe.Werte.Add(m_Asat)
            End If

            m_Alat.Gruppe = m_ActiveAlatGruppe
            m_Asat.Gruppe = m_ActiveAlatGruppe
            m_ActiveAlatGruppe.WasChanged = True
        End If

        Dim NewFibrinogenGruppe As LaborTextGruppe = Nothing
        If m_Gruppen.IndexOf(m_EntzndungsparameterGruppe) < m_Gruppen.IndexOf(m_GerinnungsGruppe) Then
            If m_ActiveAlatGruppe IsNot m_EntzndungsparameterGruppe Then
                NewFibrinogenGruppe = m_EntzndungsparameterGruppe
            End If
        Else
            If m_ActiveAlatGruppe IsNot m_GerinnungsGruppe Then
                NewFibrinogenGruppe = m_GerinnungsGruppe
            End If
        End If

        If NewFibrinogenGruppe IsNot Nothing Then
            m_ActiveFibrinogenGruppe.Werte.Remove(m_Fibrinogen)
            m_ActiveFibrinogenGruppe.WasChanged = True
            m_ActiveFibrinogenGruppe = NewFibrinogenGruppe
            m_ActiveFibrinogenGruppe.Werte.Add(m_Fibrinogen)
            m_Fibrinogen.Gruppe = m_ActiveFibrinogenGruppe
            m_ActiveFibrinogenGruppe.WasChanged = True
        End If
      
    End Sub

    Public Sub RaiseGruppenChanges()
        For Each Grp As LaborTextGruppe In m_Gruppen
            If Grp.WasChanged Then
                Grp.RaiseChanged()
                Grp.WasChanged = False
            End If
        Next
    End Sub

    Public Sub SetWert(ByVal Kuerzel As String, ByVal Value As String)
        Dim Found As Boolean = False
        For Each Grp As LaborTextGruppe In m_Gruppen
            For Each wt As LaborWert In Grp.Werte
                If wt.Kuerzel = Kuerzel Then
                    wt.Wert = Value
                    Found = True
                    Exit For
                End If
            Next
            If Found Then Exit For
        Next
        CreateText()
    End Sub
    Public Sub SetGroupNormal(ByVal Kuerzel As String)
        For Each Grp As LaborTextGruppe In m_Gruppen
            If Grp.Title = Kuerzel Then
                Grp.SetUnauffaellig(True)
                CreateText()
                Exit For
            End If
        Next
    End Sub
End Class
